/* Notifications & Message Tray */

$notification_banner_height: 64px;
$notification_banner_width: 34em;

// Banner notifications
.notification-banner {
  min-height: $notification_banner_height;
  width: $notification_banner_width;

  .notification-actions {
    spacing: 0;
  }

  .notification-button {
    @extend %bubble_button;
  }
}

// counter
.summary-source-counter {
  font-size: $base_font_size - 1pt;
  font-weight: bold;
  height: 1.6em;
  width: 1.6em;
  -shell-counter-overlap-x: 3px;
  -shell-counter-overlap-y: 3px;
  background-color: $selected_bg_color;
  color: $selected_fg_color;
  border: 2px solid $fg_color;
  box-shadow: 0 2px 2px rgba(0,0,0,0.5);
  border-radius: 0.9em; // should be 0.8 but whatever; wish I could do 50%;
}

// chat bubbles
.chat-body { spacing: 5px; }
.chat-response { margin: 5px; }
.chat-log-message { color: darken($fg_color,10%); }
.chat-new-group { padding-top: 1em; }
.chat-received {
  padding-left: 4px;
  &:rtl { padding-left: 0px; padding-right: 4px; }
}

.chat-sent {
  padding-left: 18pt;
  color: lighten($fg_color, 15%);
  &:rtl { padding-left: 0; padding-right: 18pt; }
}

.chat-meta-message {
  padding-left: 4px;
  @include fontsize($base_font_size - 2);
  font-weight: bold;
  color: lighten($fg_color,18%);
  &:rtl { padding-left: 0; padding-right: 4px; }
}
