/* This stylesheet is generated, DO NOT EDIT */
/* Copyright 2009, 2015 Red Hat, Inc.
 *
 * Portions adapted from Mx's data/style/default.css
 *   Copyright 2009 Intel Corporation
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 */
/* Global Values */
stage {
  font-size: 11pt;
  color: #eeeeec; }

/* Common Stylings */
.search-statustext {
  font-size: 2em;
  font-weight: bold;
  color: #eeeeec; }

.workspace-switcher-container, .switcher-list, .resize-popup, .osd-window {
  color: #eeeeec;
  background-color: rgba(32, 32, 32, 0.96);
  border: 1px solid rgba(255, 255, 255, 0.16);
  border-radius: 14px;
  padding: 12px; }

.workspace-thumbnails, #dash {
  color: #eeeeec;
  background-color: rgba(32, 32, 32, 0.76);
  border: 1px solid rgba(255, 255, 255, 0.16); }

.show-apps .overview-icon, .app-well-app .overview-icon,
.grid-search-result .overview-icon, .list-search-result, .search-provider-icon {
  border-radius: 9px;
  padding: 6px;
  border: 2px solid transparent;
  transition-duration: 100ms;
  text-align: center; }

.modal-dialog {
  color: #eeeeec;
  background-color: #353535;
  border: 1px solid #202020; }

.app-folder-dialog .folder-name-container .edit-folder-button, .button {
  border-radius: 5px;
  border-style: solid;
  border-width: 1px;
  min-height: 22px;
  padding: 3px 24px;
  color: #eeeeec;
  background-color: #3c3c3c;
  border-color: #2b2b2b;
  box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
  text-shadow: 0 1px rgba(0, 0, 0, 0.2);
  icon-shadow: 0 1px rgba(0, 0, 0, 0.2); }
  .app-folder-dialog .folder-name-container .edit-folder-button:focus, .button:focus {
    color: #eeeeec;
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
    box-shadow: inset 0 0 0 2px rgba(27, 106, 203, 0.6); }
  .app-folder-dialog .folder-name-container .edit-folder-button:hover, .button:hover {
    color: #eeeeec;
    background-color: #424242;
    border-color: #2b2b2b;
    box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2); }
  .app-folder-dialog .folder-name-container .edit-folder-button:insensitive, .button:insensitive {
    color: #919190;
    border-color: #252526;
    background-color: #323132;
    box-shadow: none;
    text-shadow: none;
    icon-shadow: none; }
  .app-folder-dialog .folder-name-container .edit-folder-button:active, .button:active {
    color: #eeeeec;
    background-color: #2d2d2d;
    border-color: #191919;
    text-shadow: none;
    icon-shadow: none;
    box-shadow: none; }

.modal-dialog .modal-dialog-linked-button, .hotplug-notification-item, .notification-banner .notification-button {
  color: #eeeeec;
  background-color: #3c3c3c;
  border-color: #2b2b2b;
  box-shadow: none;
  text-shadow: 0 1px rgba(0, 0, 0, 0.2);
  icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
  padding: 12px;
  border-style: solid;
  border-width: 1px;
  border-left-width: 0;
  border-bottom-width: 0; }
  .modal-dialog .modal-dialog-linked-button:insensitive, .hotplug-notification-item:insensitive, .notification-banner .notification-button:insensitive {
    color: #919190;
    border-color: #252526;
    background-color: #323132;
    box-shadow: none;
    text-shadow: none;
    icon-shadow: none; }
  .modal-dialog .modal-dialog-linked-button:hover, .hotplug-notification-item:hover, .notification-banner .notification-button:hover {
    color: #eeeeec;
    background-color: #424242;
    border-color: #2b2b2b;
    box-shadow: none;
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2); }
  .modal-dialog .modal-dialog-linked-button:focus, .hotplug-notification-item:focus, .notification-banner .notification-button:focus {
    color: #eeeeec;
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
    box-shadow: inset 0 0 0 2px rgba(27, 106, 203, 0.6); }
  .modal-dialog .modal-dialog-linked-button:active, .hotplug-notification-item:active, .notification-banner .notification-button:active {
    color: #eeeeec;
    background-color: #2d2d2d;
    border-color: #191919;
    text-shadow: none;
    icon-shadow: none;
    box-shadow: none; }
  .modal-dialog .modal-dialog-linked-button:first-child, .hotplug-notification-item:first-child, .notification-banner .notification-button:first-child {
    border-radius: 0 0 0 8px; }
  .modal-dialog .modal-dialog-linked-button:last-child, .hotplug-notification-item:last-child, .notification-banner .notification-button:last-child {
    border-right-width: 0;
    border-radius: 0 0 8px 0; }
  .modal-dialog .modal-dialog-linked-button:first-child:last-child, .hotplug-notification-item:first-child:last-child, .notification-banner .notification-button:first-child:last-child {
    border-radius: 0 0 8px 8px; }

/* WIDGETS */
.shell-link {
  color: #629fea; }
  .shell-link:hover {
    color: #8fbbf0; }

.lowres-icon {
  icon-shadow: 0 1px 2px rgba(0, 0, 0, 0.3); }

.icon-dropshadow {
  icon-shadow: 0 1px 2px rgba(0, 0, 0, 0.4); }

/* Entries */
StEntry {
  border-radius: 5px;
  padding: 8px;
  border-width: 1px;
  color: #eeeeec;
  background-color: #2d2c2e;
  border-color: #202020;
  selection-background-color: #1b6acb;
  selected-color: #fff; }
  StEntry:focus {
    border-color: #092444;
    box-shadow: inset 0 0 0 1px #1b6acb; }
  StEntry:insensitive {
    color: #919190;
    border-color: #323132;
    box-shadow: none; }
  StEntry StIcon.capslock-warning {
    icon-size: 16px;
    warning-color: #f57900;
    padding: 0 4px; }
  StEntry StIcon.peek-password {
    icon-size: 1.09em;
    padding: 0 4px; }
  StEntry StLabel.hint-text {
    margin-left: 2px;
    color: rgba(238, 238, 236, 0.7); }

/* Buttons */
/* Check Boxes */
.check-box StBoxLayout {
  spacing: .8em; }

.check-box StBin {
  width: 24px;
  height: 22px;
  background-image: url("resource:///org/gnome/shell/theme/checkbox-off.svg"); }

.check-box:focus StBin {
  background-image: url("resource:///org/gnome/shell/theme/checkbox-off-focused.svg"); }

.check-box:checked StBin {
  background-image: url("resource:///org/gnome/shell/theme/checkbox.svg"); }

.check-box:focus:checked StBin {
  background-image: url("resource:///org/gnome/shell/theme/checkbox-focused.svg"); }

/* Switches */
.toggle-switch {
  color: #eeeeec;
  height: 22px;
  width: 46px;
  background-size: contain;
  background-image: url("resource:///org/gnome/shell/theme/toggle-off-dark.svg"); }
  .toggle-switch:checked {
    background-image: url("resource:///org/gnome/shell/theme/toggle-on-dark.svg"); }

/* Slider */
.slider {
  height: 15px;
  -barlevel-height: 3px;
  -barlevel-background-color: #202020;
  -barlevel-border-width: 1px;
  -barlevel-border-color: #202020;
  -barlevel-active-background-color: #1b6acb;
  -barlevel-active-border-color: #1c6fd4;
  -barlevel-overdrive-color: #b2161d;
  -barlevel-overdrive-border-color: #bc171e;
  -barlevel-overdrive-separator-width: 1px;
  -slider-handle-radius: 7.5px;
  -slider-handle-border-width: 1px;
  -slider-handle-border-color: #eeeeec;
  color: #eeeeec; }
  .slider:hover {
    color: #424242; }
  .slider:active {
    color: #1e1e1e; }

/* Scrollbars */
StScrollView.vfade {
  -st-vfade-offset: 68px; }

StScrollView.hfade {
  -st-hfade-offset: 68px; }

StScrollBar {
  padding: 0; }
  StScrollView StScrollBar {
    min-width: 14px;
    min-height: 14px; }
  StScrollBar StBin#trough {
    border-radius: 0;
    background-color: transparent; }
  StScrollBar StButton#vhandle, StScrollBar StButton#hhandle {
    border-radius: 8px;
    background-color: #a4a4a3;
    margin: 3px; }
    StScrollBar StButton#vhandle:hover, StScrollBar StButton#hhandle:hover {
      background-color: #c9c9c7; }
    StScrollBar StButton#vhandle:active, StScrollBar StButton#hhandle:active {
      background-color: #1b6acb; }

/* Popovers/Menus */
.popup-menu-boxpointer, .candidate-popup-boxpointer {
  -arrow-border-radius: 9px;
  -arrow-background-color: #353535;
  -arrow-border-width: 1px;
  -arrow-border-color: #202020;
  -arrow-base: 24px;
  -arrow-rise: 12px;
  -arrow-box-shadow: 0 1px 3px rgba(0, 0, 0, 0.5); }

.popup-menu {
  min-width: 15em;
  color: #eeeeec; }
  .popup-menu.panel-menu {
    -boxpointer-gap: 4px;
    margin-bottom: 1.75em; }

.popup-menu-content {
  padding: 16px 0; }

.popup-menu-item {
  spacing: 6px;
  padding: 6px; }
  .popup-menu-item:ltr {
    padding-right: 1.75em;
    padding-left: 0; }
  .popup-menu-item:rtl {
    padding-right: 0;
    padding-left: 1.75em; }
  .popup-menu-item:checked {
    background-color: #3a3a3a;
    box-shadow: none; }
  .popup-menu-item.selected {
    background-color: rgba(255, 255, 255, 0.1);
    color: #eeeeec; }
  .popup-menu-item:active {
    background-color: #1b6acb;
    color: #fff; }
  .popup-menu-item:insensitive {
    color: rgba(238, 238, 236, 0.5); }

.popup-inactive-menu-item {
  color: #eeeeec; }
  .popup-inactive-menu-item:insensitive {
    color: rgba(238, 238, 236, 0.5); }

.popup-menu-arrow,
.popup-menu-icon {
  icon-size: 1.09em; }

.popup-sub-menu {
  background-color: #2d2d2d;
  box-shadow: none;
  border-top: 1px solid rgba(32, 32, 32, 0.8);
  border-bottom: 1px solid rgba(32, 32, 32, 0.8); }

.popup-menu-ornament {
  width: 1.2em; }
  .popup-menu-ornament:ltr {
    text-align: right; }
  .popup-menu-ornament:rtl {
    text-align: left; }

.popup-separator-menu-item {
  padding: 0; }
  .popup-separator-menu-item .popup-separator-menu-item-separator {
    height: 1px;
    margin: 6px 64px;
    background-color: #262626; }
    .popup-sub-menu .popup-separator-menu-item .popup-separator-menu-item-separator {
      margin: 0 64px 0 32px;
      background-color: #4e4e4e; }

.background-menu {
  -boxpointer-gap: 4px;
  -arrow-rise: 0px; }

.aggregate-menu {
  min-width: 21em; }
  .aggregate-menu .popup-menu-icon {
    padding: 0;
    margin: 0 4px;
    -st-icon-style: symbolic; }
  .aggregate-menu .popup-sub-menu .popup-menu-item > :first-child:ltr {
    padding-left: 14px;
    margin-left: 1.09em; }
  .aggregate-menu .popup-sub-menu .popup-menu-item > :first-child:rtl {
    padding-right: 14px;
    margin-right: 1.09em; }

/* Date/Time Menu */
.clock-display-box {
  spacing: 2px; }
  .clock-display-box .clock {
    padding-left: 12px;
    padding-right: 12px; }

#calendarArea {
  padding: 0; }

.datemenu-calendar-column {
  spacing: 6px;
  border: 0 solid #2d2d2d;
  padding: 0 12px; }
  .datemenu-calendar-column:ltr {
    margin-right: 8px;
    border-left-width: 1px; }
  .datemenu-calendar-column:rtl {
    margin-left: 8px;
    border-right-width: 1px; }
  .datemenu-calendar-column .datemenu-displays-box {
    spacing: 6px; }

.events-section-title {
  border-width: 1px;
  border-style: solid;
  border-radius: 7px;
  margin: 4px;
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0), 0 1px rgba(238, 238, 236, 0);
  text-shadow: none;
  icon-shadow: none;
  color: #878787;
  font-weight: bold;
  padding: .4em; }
  .events-section-title:focus {
    color: #eeeeec;
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
    box-shadow: inset 0 0 0 2px rgba(27, 106, 203, 0.6); }
  .events-section-title:hover {
    color: #eeeeec;
    background-color: #424242;
    border-color: #2b2b2b;
    box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2); }
  .events-section-title:active {
    color: #eeeeec;
    background-color: #2d2d2d;
    border-color: #191919;
    text-shadow: none;
    icon-shadow: none;
    box-shadow: none; }

/* today button (the date) */
.datemenu-today-button {
  border-width: 1px;
  border-style: solid;
  border-radius: 7px;
  margin: 4px;
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0), 0 1px rgba(238, 238, 236, 0);
  text-shadow: none;
  icon-shadow: none;
  padding: 9px; }
  .datemenu-today-button:focus {
    color: #eeeeec;
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
    box-shadow: inset 0 0 0 2px rgba(27, 106, 203, 0.6); }
  .datemenu-today-button:hover {
    color: #eeeeec;
    background-color: #424242;
    border-color: #2b2b2b;
    box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2); }
  .datemenu-today-button:active {
    color: #eeeeec;
    background-color: #2d2d2d;
    border-color: #191919;
    text-shadow: none;
    icon-shadow: none;
    box-shadow: none; }
  .datemenu-today-button .day-label {
    font-size: 12pt;
    font-weight: bold; }
  .datemenu-today-button .date-label {
    font-size: 18pt;
    font-weight: 1000; }

/* Calendar */
.calendar {
  border-width: 1px;
  border-style: solid;
  border-radius: 7px;
  margin: 4px;
  color: #eeeeec;
  background-color: #3c3c3c;
  border-color: #2b2b2b;
  box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
  text-shadow: 0 1px rgba(0, 0, 0, 0.2);
  icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
  padding: 6px; }
  .calendar:focus {
    color: #eeeeec;
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
    box-shadow: inset 0 0 0 2px rgba(27, 106, 203, 0.6); }
  .calendar:hover {
    color: #eeeeec;
    background-color: #424242;
    border-color: #2b2b2b;
    box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2); }
  .calendar:active {
    color: #eeeeec;
    background-color: #2d2d2d;
    border-color: #191919;
    text-shadow: none;
    icon-shadow: none;
    box-shadow: none; }
  .calendar .calendar-month-label {
    color: #fafaf9;
    font-weight: bold;
    padding: 8px 0; }
  .calendar .calendar-change-month-back StIcon,
  .calendar .calendar-change-month-forward StIcon {
    icon-size: 1.09em; }
  .calendar .pager-button {
    background-color: transparent;
    height: 32px;
    width: 32px;
    border-radius: 5px; }
    .calendar .pager-button:hover, .calendar .pager-button:focus {
      background-color: #4e4e4e; }
    .calendar .pager-button:active {
      background-color: #1e1e1e; }
  .calendar .calendar-day-base {
    font-size: 8pt;
    text-align: center;
    width: 32px;
    height: 32px;
    padding: 0;
    margin: 2px;
    border-radius: 18px;
    border: 1px solid transparent;
    font-feature-settings: "tnum"; }
    .calendar .calendar-day-base:hover, .calendar .calendar-day-base:focus {
      background-color: #424242; }
    .calendar .calendar-day-base:active, .calendar .calendar-day-base:selected {
      color: white;
      background-color: #282828; }
    .calendar .calendar-day-base.calendar-day-heading {
      color: white;
      margin-top: 1em;
      font-size: 7pt; }
  .calendar .calendar-day {
    border-width: 0; }
  .calendar .calendar-day-top {
    border-top-width: 1px; }
  .calendar .calendar-day-left {
    border-left-width: 1px; }
  .calendar .calendar-nonwork-day {
    color: #919190; }
  .calendar .calendar-today {
    font-weight: bold;
    border: 1px solid transparent;
    background-color: #1b6acb;
    color: #fff; }
    .calendar .calendar-today:hover, .calendar .calendar-today:focus {
      background-color: #1d72d8;
      color: #fff; }
    .calendar .calendar-today:active, .calendar .calendar-today:selected {
      background-color: #1b6acb;
      color: #fff; }
      .calendar .calendar-today:active:hover, .calendar .calendar-today:active:focus, .calendar .calendar-today:selected:hover, .calendar .calendar-today:selected:focus {
        background-color: #1d72d8;
        color: #fff; }
  .calendar .calendar-day-with-events {
    background-image: url("resource:///org/gnome/shell/theme/calendar-today.svg"); }
    .calendar .calendar-day-with-events.calendar-work-day {
      color: white;
      font-weight: bold; }
  .calendar .calendar-other-month-day {
    color: rgba(238, 238, 236, 0.5); }
  .calendar .calendar-week-number {
    font-size: 7pt;
    font-weight: bold;
    height: 1.8em;
    width: 2.3em;
    border-radius: 2px;
    margin: 6px;
    background-color: #303030;
    color: #fafaf9; }

/* Events */
.events-button {
  border-width: 1px;
  border-style: solid;
  border-radius: 7px;
  margin: 4px;
  color: #eeeeec;
  background-color: #3c3c3c;
  border-color: #2b2b2b;
  box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
  text-shadow: 0 1px rgba(0, 0, 0, 0.2);
  icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
  padding: 12px; }
  .events-button:focus {
    color: #eeeeec;
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
    box-shadow: inset 0 0 0 2px rgba(27, 106, 203, 0.6); }
  .events-button:hover {
    color: #eeeeec;
    background-color: #424242;
    border-color: #2b2b2b;
    box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2); }
  .events-button:active {
    color: #eeeeec;
    background-color: #2d2d2d;
    border-color: #191919;
    text-shadow: none;
    icon-shadow: none;
    box-shadow: none; }
  .events-button .events-box {
    spacing: 6px; }
  .events-button .events-list {
    spacing: 12px; }
  .events-button .events-title {
    color: #878787;
    font-weight: bold;
    margin-bottom: 4px; }
  .events-button .event-time {
    color: #bebeb6;
    font-feature-settings: "tnum";
    font-size: 10pt; }

/* World clocks */
.world-clocks-button {
  border-width: 1px;
  border-style: solid;
  border-radius: 7px;
  margin: 4px;
  color: #eeeeec;
  background-color: #3c3c3c;
  border-color: #2b2b2b;
  box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
  text-shadow: 0 1px rgba(0, 0, 0, 0.2);
  icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
  padding: 12px; }
  .world-clocks-button:focus {
    color: #eeeeec;
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
    box-shadow: inset 0 0 0 2px rgba(27, 106, 203, 0.6); }
  .world-clocks-button:hover {
    color: #eeeeec;
    background-color: #424242;
    border-color: #2b2b2b;
    box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2); }
  .world-clocks-button:active {
    color: #eeeeec;
    background-color: #2d2d2d;
    border-color: #191919;
    text-shadow: none;
    icon-shadow: none;
    box-shadow: none; }
  .world-clocks-button .world-clocks-grid {
    spacing-rows: 6px;
    spacing-columns: 12px; }
  .world-clocks-button .world-clocks-header {
    color: #878787;
    font-weight: bold; }
  .world-clocks-button .world-clocks-city {
    color: #eeeeec;
    font-size: 11pt;
    font-weight: normal; }
  .world-clocks-button .world-clocks-time {
    font-weight: bold;
    color: #eeeeec;
    font-feature-settings: "tnum";
    font-size: 11pt; }
    .world-clocks-button .world-clocks-time:ltr {
      text-align: right; }
    .world-clocks-button .world-clocks-time:rtl {
      text-align: left; }
  .world-clocks-button .world-clocks-timezone {
    color: #bebeb6;
    font-feature-settings: "tnum";
    font-size: 10pt; }

/* Weather */
.weather-button {
  border-width: 1px;
  border-style: solid;
  border-radius: 7px;
  margin: 4px;
  color: #eeeeec;
  background-color: #3c3c3c;
  border-color: #2b2b2b;
  box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
  text-shadow: 0 1px rgba(0, 0, 0, 0.2);
  icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
  padding: 12px; }
  .weather-button:focus {
    color: #eeeeec;
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
    box-shadow: inset 0 0 0 2px rgba(27, 106, 203, 0.6); }
  .weather-button:hover {
    color: #eeeeec;
    background-color: #424242;
    border-color: #2b2b2b;
    box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2); }
  .weather-button:active {
    color: #eeeeec;
    background-color: #2d2d2d;
    border-color: #191919;
    text-shadow: none;
    icon-shadow: none;
    box-shadow: none; }
  .weather-button .weather-box {
    spacing: 10px; }
  .weather-button .weather-header-box {
    spacing: 6px; }
  .weather-button .weather-header {
    color: #878787;
    font-weight: bold; }
    .weather-button .weather-header.location {
      font-weight: normal;
      font-size: 10pt; }
  .weather-button .weather-grid {
    spacing-rows: 6px;
    spacing-columns: 12px; }
  .weather-button .weather-forecast-time {
    color: #a6a69b;
    font-feature-settings: "tnum";
    font-size: 9pt;
    font-weight: normal;
    padding-top: 0.2em;
    padding-bottom: 0.4em; }
  .weather-button .weather-forecast-icon {
    icon-size: 2.18em; }
  .weather-button .weather-forecast-temp {
    font-weight: bold; }

/* Message List */
.message-list {
  width: 31.5em;
  padding: 0 12px; }
  .message-list .message-list-placeholder {
    spacing: 12px; }

.message-list-sections {
  spacing: 6px;
  margin: 0 16px; }

.message-list-section,
.message-list-section-list {
  spacing: 6px; }

.message-list-controls {
  margin: 8px 16px 0;
  padding: 4px;
  spacing: 12px; }

.message {
  border-width: 1px;
  border-style: solid;
  border-radius: 7px;
  margin: 4px;
  color: #eeeeec;
  background-color: #3c3c3c;
  border-color: #2b2b2b;
  box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
  text-shadow: 0 1px rgba(0, 0, 0, 0.2);
  icon-shadow: 0 1px rgba(0, 0, 0, 0.2); }
  .message:focus {
    color: #eeeeec;
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
    box-shadow: inset 0 0 0 2px rgba(27, 106, 203, 0.6); }
  .message:hover {
    color: #eeeeec;
    background-color: #424242;
    border-color: #2b2b2b;
    box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2); }
  .message:active {
    color: #eeeeec;
    background-color: #2d2d2d;
    border-color: #191919;
    text-shadow: none;
    icon-shadow: none;
    box-shadow: none; }
  .message .message-icon-bin {
    padding: 18px 0 18px 12px; }
    .message .message-icon-bin:rtl {
      padding: 18px 12px 18px 0; }
    .message .message-icon-bin > StIcon {
      icon-size: 2.18em;
      -st-icon-style: symbolic; }
    .message .message-icon-bin > .fallback-app-icon {
      width: 1.09em;
      height: 1.09em; }
  .message .message-content {
    padding: 14px;
    spacing: 4px; }
  .message .message-title {
    font-weight: bold; }
  .message .message-secondary-bin {
    padding: 0 8px; }
    .message .message-secondary-bin > .event-time {
      color: rgba(238, 238, 236, 0.5);
      font-size: 9pt;
      /* HACK: the label should be baseline-aligned with a 1em label, fake this with some bottom padding */
      padding-bottom: 0.13em; }
      .message .message-secondary-bin > .event-time:ltr {
        text-align: right; }
      .message .message-secondary-bin > .event-time:rtl {
        text-align: left; }
  .message .message-close-button {
    color: white; }
    .message .message-close-button:hover {
      color: #d6d6d1; }
    .message .message-close-button:active {
      color: #bebeb6; }
  .message .message-body {
    color: #d6d6d1; }

.url-highlighter {
  link-color: #629fea; }

/* Media Controls */
.message-media-control {
  padding: 12px 24px;
  color: #cacac4; }
  .message-media-control:hover {
    background-color: #4e4e4e;
    color: #eeeeec; }
  .message-media-control:active {
    background-color: #3c3c3c;
    color: #eeeeec; }
  .message-media-control:insensitive {
    color: #8e8e80; }
  .message-media-control:last-child:ltr {
    border-radius: 0 7px 7px 0; }
  .message-media-control:last-child:rtl {
    border-radius: 7px 0 0 7px; }

.media-message-cover-icon {
  icon-size: 2.18em !important;
  border-radius: 5px; }
  .media-message-cover-icon.fallback {
    color: #c5c5be;
    background-color: #353535;
    border: 1px solid transparent;
    border-radius: 5px;
    icon-size: 2.18em !important; }

.candidate-popup-content {
  padding: 0.5em;
  spacing: 0.3em; }

.candidate-index {
  padding: 0 0.5em 0 0;
  color: #d6d6d1; }

.candidate-box {
  padding: 0.3em 0.5em 0.3em 0.5em;
  border-radius: 5px; }
  .candidate-box:selected, .candidate-box:hover {
    background-color: #1b6acb;
    color: #fff; }

.candidate-page-button-box {
  height: 2em; }
  .vertical .candidate-page-button-box {
    padding-top: 0.5em; }
  .horizontal .candidate-page-button-box {
    padding-left: 0.5em; }

.candidate-page-button {
  padding: 4px; }

.candidate-page-button-previous {
  border-radius: 5px 0px 0px 5px;
  border-right-width: 0; }

.candidate-page-button-next {
  border-radius: 0px 5px 5px 0px; }

.candidate-page-button-icon {
  icon-size: 1em; }

/* Notifications & Message Tray */
.notification-banner {
  min-height: 64px;
  width: 34em; }
  .notification-banner .notification-actions {
    spacing: 0; }

.summary-source-counter {
  font-size: 10pt;
  font-weight: bold;
  height: 1.6em;
  width: 1.6em;
  -shell-counter-overlap-x: 3px;
  -shell-counter-overlap-y: 3px;
  background-color: #1b6acb;
  color: #fff;
  border: 2px solid #eeeeec;
  box-shadow: 0 2px 2px rgba(0, 0, 0, 0.5);
  border-radius: 0.9em; }

.chat-body {
  spacing: 5px; }

.chat-response {
  margin: 5px; }

.chat-log-message {
  color: #d6d6d1; }

.chat-new-group {
  padding-top: 1em; }

.chat-received {
  padding-left: 4px; }
  .chat-received:rtl {
    padding-left: 0px;
    padding-right: 4px; }

.chat-sent {
  padding-left: 18pt;
  color: white; }
  .chat-sent:rtl {
    padding-left: 0;
    padding-right: 18pt; }

.chat-meta-message {
  padding-left: 4px;
  font-size: 9pt;
  font-weight: bold;
  color: white; }
  .chat-meta-message:rtl {
    padding-left: 0;
    padding-right: 4px; }

.hotplug-notification-item-icon {
  icon-size: 24px;
  padding: 0 4px; }

/* Modal Dialogs */
.headline {
  font-size: 12pt; }

.modal-dialog {
  border-radius: 10px; }
  .modal-dialog .modal-dialog-content-box {
    margin: 32px 40px;
    spacing: 32px;
    max-width: 28em; }

/* End Session Dialog */
.end-session-dialog {
  width: 30em; }
  .end-session-dialog .end-session-dialog-battery-warning,
  .end-session-dialog .dialog-list-title {
    color: #f57900; }

/* Message Dialog */
.message-dialog-content {
  spacing: 18px; }
  .message-dialog-content .message-dialog-title {
    text-align: center;
    font-size: 18pt;
    font-weight: 800; }
    .message-dialog-content .message-dialog-title.lightweight {
      font-size: 13pt;
      font-weight: 800; }
  .message-dialog-content .message-dialog-description {
    text-align: center; }

/* Dialog List */
.dialog-list {
  spacing: 18px; }
  .dialog-list .dialog-list-title {
    text-align: center;
    font-weight: bold; }
  .dialog-list .dialog-list-scrollview {
    max-height: 200px; }
  .dialog-list .dialog-list-box {
    spacing: 1em; }
    .dialog-list .dialog-list-box .dialog-list-item {
      spacing: 1em; }
      .dialog-list .dialog-list-box .dialog-list-item .dialog-list-item-title {
        font-weight: bold; }
      .dialog-list .dialog-list-box .dialog-list-item .dialog-list-item-description {
        color: #e2e2df;
        font-size: 10pt; }

/* Run Dialog */
.run-dialog .modal-dialog-content-box {
  margin-top: 24px;
  margin-bottom: 14px; }

.run-dialog .run-dialog-entry {
  width: 20em; }

.run-dialog .run-dialog-description {
  font-size: 10pt;
  text-align: center;
  color: #bebeb6; }

/* Password or Authentication Dialog */
.prompt-dialog {
  width: 28em; }
  .prompt-dialog .modal-dialog-content-box {
    margin-bottom: 24px; }

.prompt-dialog-password-grid {
  spacing-rows: 8px;
  spacing-columns: 4px; }
  .prompt-dialog-password-grid .prompt-dialog-password-entry {
    width: auto; }
    .prompt-dialog-password-grid .prompt-dialog-password-entry:ltr {
      margin-left: 20px; }
    .prompt-dialog-password-grid .prompt-dialog-password-entry:rtl {
      margin-right: 20px; }

.prompt-dialog-password-layout {
  spacing: 8px; }

.prompt-dialog-password-entry {
  width: 20em; }

.prompt-dialog-error-label,
.prompt-dialog-info-label,
.prompt-dialog-null-label {
  text-align: center;
  font-size: 10pt; }

.prompt-dialog-error-label {
  color: #f57900; }

/* Polkit Dialog */
.polkit-dialog-user-layout {
  text-align: center;
  spacing: 8px;
  margin-bottom: 6px; }
  .polkit-dialog-user-layout .polkit-dialog-user-root-label {
    color: #f57900; }

/* Audio selection dialog */
.audio-device-selection-dialog .modal-dialog-content-box {
  margin-bottom: 28px; }

.audio-device-selection-dialog .audio-selection-box {
  spacing: 20px; }

.audio-selection-device {
  border: 1px solid #2d2d2d;
  border-radius: 12px; }
  .audio-selection-device:hover, .audio-selection-device:focus {
    background-color: #424242; }
  .audio-selection-device:active {
    background-color: #1b6acb;
    color: #fff; }

.audio-selection-device-box {
  padding: 20px;
  spacing: 20px; }

.audio-selection-device-icon {
  icon-size: 4.36em; }

/* Network Dialogs */
.nm-dialog {
  max-height: 34em;
  min-height: 31em;
  min-width: 32em; }

.nm-dialog-content {
  spacing: 20px;
  padding: 24px; }

.nm-dialog-airplane-box {
  spacing: 12px; }

.nm-dialog-airplane-headline {
  font-weight: bold;
  text-align: center; }

.nm-dialog-airplane-text {
  color: #eeeeec; }

.nm-dialog-header {
  font-weight: bold; }

.nm-dialog-header-icon {
  icon-size: 2.18em; }

.nm-dialog-header-hbox {
  spacing: 10px; }

.nm-dialog-scroll-view {
  border: 1px solid #202020;
  padding: 0;
  background-color: #2d2d2d; }

.nm-dialog-item {
  font-size: 11pt;
  border-bottom: 1px solid #202020;
  padding: 12px;
  spacing: 0px; }
  .nm-dialog-item:selected {
    background-color: #1b6acb;
    color: #fff; }

.nm-dialog-icon {
  icon-size: 1.09em; }

.nm-dialog-icons {
  spacing: 12px; }

.no-networks-label {
  color: #919190; }

.no-networks-box {
  spacing: 6px; }

/* OSD */
.osd-window {
  text-align: center;
  font-weight: bold;
  spacing: 12px;
  margin: 32px;
  min-width: 64px;
  min-height: 64px; }
  .osd-window StIcon {
    icon-size: 6.54em; }
  .osd-window .osd-monitor-label {
    font-size: 3em; }
  .osd-window .level {
    height: 8px;
    -barlevel-height: 8px;
    -barlevel-background-color: rgba(238, 238, 236, 0.1);
    -barlevel-active-background-color: #eeeeec;
    -barlevel-overdrive-color: #b2161d;
    -barlevel-overdrive-separator-width: 3px; }

/* Pad OSD */
.pad-osd-window {
  padding: 32px;
  background-color: rgba(0, 0, 0, 0.8); }
  .pad-osd-window .pad-osd-title-box {
    spacing: 12px; }
  .pad-osd-window .pad-osd-title-menu-box {
    spacing: 6px; }

.combo-box-label {
  width: 15em; }

/* App Switcher */
.switcher-popup {
  padding: 8px;
  spacing: 24px; }

.switcher-list .item-box {
  padding: 8px;
  border-radius: 6px;
  border: 1px solid transparent; }
  .switcher-list .item-box:outlined {
    background-color: rgba(238, 238, 236, 0.3); }
  .switcher-list .item-box:selected {
    background-color: rgba(238, 238, 236, 0.3);
    color: #eeeeec; }

.switcher-list .thumbnail-box {
  padding: 2px;
  spacing: 6px; }

.switcher-list .thumbnail {
  width: 256px; }

.switcher-list .separator {
  width: 1px;
  background: #202020; }

.switcher-list .switcher-list-item-container {
  spacing: 12px; }

.switcher-arrow {
  border-color: rgba(0, 0, 0, 0);
  color: rgba(238, 238, 236, 0.8); }
  .switcher-arrow:highlighted {
    color: #eeeeec; }

.input-source-switcher-symbol {
  font-size: 34pt;
  width: 96px;
  height: 96px; }

.cycler-highlight {
  border: 5px solid #1b6acb; }

/* Workspace Switcher */
.workspace-switcher-group {
  padding: 12px; }

.workspace-switcher {
  background: transparent;
  border: none;
  border-radius: 0;
  padding: 0;
  spacing: 12px; }

.ws-switcher-box {
  background: transparent;
  height: 50px;
  background-size: 32px;
  border: 1px solid rgba(238, 238, 236, 0.1);
  border-radius: 8px; }

.ws-switcher-active-up,
.ws-switcher-active-down,
.ws-switcher-active-left,
.ws-switcher-active-right {
  height: 52px;
  background-color: #1b6acb;
  border: 1px solid #1f76e1;
  border-radius: 8px;
  color: #fff; }

/* Top Bar */
#panel {
  background-color: #000;
  font-weight: bold;
  height: 2.2em;
  font-feature-settings: "tnum";
  transition-duration: 250ms; }
  #panel.unlock-screen, #panel.login-screen, #panel:overview {
    background-color: transparent; }
    #panel.unlock-screen .panel-corner, #panel.login-screen .panel-corner, #panel:overview .panel-corner {
      -panel-corner-opacity: 0; }
  #panel #panelLeft, #panel #panelCenter {
    spacing: 6px; }
  #panel .panel-corner {
    -panel-corner-radius: 6px;
    -panel-corner-background-color: #000;
    -panel-corner-border-width: 2px;
    -panel-corner-border-color: transparent;
    -panel-corner-opacity: 1;
    transition-duration: 250ms; }
  #panel .panel-button {
    font-weight: bold;
    color: #ccc;
    -natural-hpadding: 12px;
    -minimum-hpadding: 6px;
    transition-duration: 150ms;
    border: 3px solid transparent;
    border-radius: 99px; }
    #panel .panel-button.clock-display .clock {
      transition-duration: 150ms;
      border: 3px solid transparent;
      border-radius: 99px; }
    #panel .panel-button:hover, #panel .panel-button:active, #panel .panel-button:overview, #panel .panel-button:focus, #panel .panel-button:checked {
      box-shadow: inset 0 0 0 100px rgba(255, 255, 255, 0.2); }
      #panel .panel-button:hover.clock-display, #panel .panel-button:active.clock-display, #panel .panel-button:overview.clock-display, #panel .panel-button:focus.clock-display, #panel .panel-button:checked.clock-display {
        box-shadow: none; }
        #panel .panel-button:hover.clock-display .clock, #panel .panel-button:active.clock-display .clock, #panel .panel-button:overview.clock-display .clock, #panel .panel-button:focus.clock-display .clock, #panel .panel-button:checked.clock-display .clock {
          box-shadow: inset 0 0 0 100px rgba(255, 255, 255, 0.2); }
    #panel .panel-button .system-status-icon {
      icon-size: 1.09em;
      padding: 5px; }
    #panel .panel-button .app-menu-icon {
      margin-left: 4px;
      margin-right: 4px;
      -st-icon-style: symbolic; }
    #panel .panel-button#panelActivities {
      -natural-hpadding: 18px; }
  #panel.unlock-screen .panel-button, #panel.login-screen .panel-button, #panel:overview .panel-button {
    color: #f5f5f4; }
    #panel.unlock-screen .panel-button:hover, #panel.unlock-screen .panel-button:active, #panel.unlock-screen .panel-button:overview, #panel.unlock-screen .panel-button:focus, #panel.unlock-screen .panel-button:checked, #panel.login-screen .panel-button:hover, #panel.login-screen .panel-button:active, #panel.login-screen .panel-button:overview, #panel.login-screen .panel-button:focus, #panel.login-screen .panel-button:checked, #panel:overview .panel-button:hover, #panel:overview .panel-button:active, #panel:overview .panel-button:overview, #panel:overview .panel-button:focus, #panel:overview .panel-button:checked {
      box-shadow: inset 0 0 0 100px rgba(255, 255, 255, 0.15); }
      #panel.unlock-screen .panel-button:hover.clock-display, #panel.unlock-screen .panel-button:active.clock-display, #panel.unlock-screen .panel-button:overview.clock-display, #panel.unlock-screen .panel-button:focus.clock-display, #panel.unlock-screen .panel-button:checked.clock-display, #panel.login-screen .panel-button:hover.clock-display, #panel.login-screen .panel-button:active.clock-display, #panel.login-screen .panel-button:overview.clock-display, #panel.login-screen .panel-button:focus.clock-display, #panel.login-screen .panel-button:checked.clock-display, #panel:overview .panel-button:hover.clock-display, #panel:overview .panel-button:active.clock-display, #panel:overview .panel-button:overview.clock-display, #panel:overview .panel-button:focus.clock-display, #panel:overview .panel-button:checked.clock-display {
        box-shadow: none; }
        #panel.unlock-screen .panel-button:hover.clock-display .clock, #panel.unlock-screen .panel-button:active.clock-display .clock, #panel.unlock-screen .panel-button:overview.clock-display .clock, #panel.unlock-screen .panel-button:focus.clock-display .clock, #panel.unlock-screen .panel-button:checked.clock-display .clock, #panel.login-screen .panel-button:hover.clock-display .clock, #panel.login-screen .panel-button:active.clock-display .clock, #panel.login-screen .panel-button:overview.clock-display .clock, #panel.login-screen .panel-button:focus.clock-display .clock, #panel.login-screen .panel-button:checked.clock-display .clock, #panel:overview .panel-button:hover.clock-display .clock, #panel:overview .panel-button:active.clock-display .clock, #panel:overview .panel-button:overview.clock-display .clock, #panel:overview .panel-button:focus.clock-display .clock, #panel:overview .panel-button:checked.clock-display .clock {
          box-shadow: inset 0 0 0 100px rgba(255, 255, 255, 0.15); }
  #panel .panel-status-indicators-box,
  #panel .panel-status-menu-box {
    spacing: 2px; }
  #panel .power-status.panel-status-indicators-box {
    spacing: 0; }
  #panel .screencast-indicator,
  #panel .remote-access-indicator {
    color: #f57900; }

#appMenu {
  spacing: 6px; }
  #appMenu .label-shadow {
    color: transparent; }

/* Activities Ripple */
.ripple-box {
  background-color: rgba(188, 214, 246, 0.3);
  box-shadow: 0 0 2px 2px #629fea;
  width: 52px;
  height: 52px;
  border-radius: 0 0 52px 0; }
  .ripple-box:rtl {
    border-radius: 0 0 0 52px; }

/* OVERVIEW */
#overview {
  spacing: 24px;
  background-color: transparent; }

.overview-controls {
  padding-bottom: 32px; }

/* Window Picker */
.window-picker {
  spacing: 6px;
  padding: 12px; }
  .window-picker.external-monitor {
    padding: 12px; }

.window-clone-border {
  border-width: 6px;
  border-style: solid;
  border-color: rgba(255, 255, 255, 0.35);
  border-radius: 7px;
  box-shadow: inset 0 0 0 1px rgba(32, 32, 32, 0.2); }

.window-caption {
  color: #eeeeec;
  background-color: rgba(32, 32, 32, 0.96);
  border: 1px solid rgba(255, 255, 255, 0.16);
  border-radius: 6px;
  padding: 6px 12px;
  font-weight: bold;
  font-size: 12pt; }

.window-close {
  background-color: #1b6acb;
  color: #fff;
  border: none;
  border-radius: 18px;
  padding: 3px;
  height: 24px;
  width: 24px;
  box-shadow: -1px 1px 5px 0px rgba(0, 0, 0, 0.5); }
  .window-close:hover {
    background-color: #1f76e1; }
  .window-close:active {
    background-color: #185fb4; }


.search-entry {
  width: 320px;
  padding: 7px 9px;
  border-radius: 18px;
  color: rgba(238, 238, 236, 0.7);
  background-color: #353535;
  border-color: #202020; }
  
  .search-entry:hover {
    background-color: #424242;
    border-color: #282828;
    color: #fafaf9; }
  
  .search-entry:focus {
    padding: 6px 8px;
    border-width: 2px;
    border-style: solid;
    border-color: #1b6acb;
    color: #eeeeec;
    box-shadow: inset 0 1px 2px 1px rgba(0, 0, 0, 0.2); }
  
  .search-entry .search-entry-icon {
    icon-size: 1.09em;
    padding: 0 4px;
    color: inherit; }

/* Search */
#searchResultsContent {
  max-width: 1024px;
  spacing: 8px; }

.search-section {
  spacing: 8px; }
  .search-section .search-section-separator {
    height: 0;
    background-color: transparent; }

.search-section-content {
  background-color: rgba(45, 45, 45, 0.76);
  border-radius: 13px;
  border: 1px solid rgba(255, 255, 255, 0.16);
  box-shadow: 0 2px 4px 0 rgba(0, 0, 0, 0.2);
  text-shadow: 0 1px rgba(0, 0, 0, 0.2);
  color: #eeeeec;
  padding: 18px;
  spacing: 8px; }

.list-search-result:focus, .search-provider-icon:focus, .list-search-result:hover, .search-provider-icon:hover, .list-search-result:selected, .search-provider-icon:selected {
  background-color: rgba(238, 238, 236, 0.1);
  transition-duration: 200ms; }

.list-search-result:active, .search-provider-icon:active, .list-search-result:checked, .search-provider-icon:checked {
  background-color: rgba(7, 7, 7, 0.86); }

.grid-search-results {
  spacing: 36px; }

.search-provider-icon .list-search-provider-content {
  spacing: 12px; }
  .search-provider-icon .list-search-provider-content .list-search-provider-details {
    width: 120px;
    margin-top: 0;
    color: #dbdbd6; }

.list-search-results {
  spacing: 6px; }

.list-search-result .list-search-result-content {
  spacing: 6px; }

.list-search-result .list-search-result-title {
  spacing: 12px; }

.list-search-result .list-search-result-description {
  color: #a6a69b; }

/* App Grid */
.icon-grid {
  row-spacing: 18px;
  column-spacing: 18px;
  max-row-spacing: 72px;
  max-column-spacing: 72px;
  page-padding-top: 36px;
  page-padding-bottom: 36px;
  page-padding-left: 36px;
  page-padding-right: 36px; }

/* App Icons */
.app-well-app .overview-icon,
.grid-search-result .overview-icon {
  color: #fff; }

.app-well-app:hover .overview-icon, .app-well-app:selected .overview-icon,
.grid-search-result:hover .overview-icon,
.grid-search-result:selected .overview-icon {
  background-color: rgba(255, 255, 255, 0.1); }

.app-well-app:focus .overview-icon,
.grid-search-result:focus .overview-icon {
  background-color: rgba(255, 255, 255, 0.3); }

.app-well-app:drop .overview-icon,
.grid-search-result:drop .overview-icon {
  background-color: rgba(27, 106, 203, 0.85); }

.app-well-app:active .overview-icon, .app-well-app:checked .overview-icon,
.grid-search-result:active .overview-icon,
.grid-search-result:checked .overview-icon {
  background-color: rgba(7, 7, 7, 0.46); }

.app-well-app .overview-icon.overview-icon-with-label,
.grid-search-result .overview-icon.overview-icon-with-label {
  padding: 10px 8px 5px 8px; }
  .app-well-app .overview-icon.overview-icon-with-label > StBoxLayout,
  .grid-search-result .overview-icon.overview-icon-with-label > StBoxLayout {
    spacing: 6px; }

/* App Folders */
.app-well-app.app-folder {
  background-color: rgba(32, 32, 32, 0.16);
  border-radius: 9px; }

.app-folder-dialog {
  border-radius: 15px;
  border: 1px solid rgba(255, 255, 255, 0.16);
  background-color: rgba(7, 7, 7, 0.91);
  padding: 12px; }
  .app-folder-dialog .folder-name-container {
    padding: 24px 36px 0;
    spacing: 12px;
    /* FIXME: this is to keep the label in sync with the entry */ }
    .app-folder-dialog .folder-name-container .folder-name-label, .app-folder-dialog .folder-name-container .folder-name-entry {
      font-size: 18pt;
      font-weight: 800; }
    .app-folder-dialog .folder-name-container .folder-name-entry {
      width: 300px; }
    .app-folder-dialog .folder-name-container .folder-name-label {
      padding: 5px 7px;
      color: #eeeeec; }
    .app-folder-dialog .folder-name-container .edit-folder-button {
      padding: 0;
      width: 36px;
      height: 36px;
      border-radius: 18px; }
      .app-folder-dialog .folder-name-container .edit-folder-button > StIcon {
        icon-size: 16px; }
  .app-folder-dialog .icon-grid {
    row-spacing: 12px;
    column-spacing: 30px;
    page-padding-top: 0;
    page-padding-bottom: 0; }
  .app-folder-dialog .page-indicators {
    margin-bottom: 18px; }

.app-folder-dialog-container {
  padding: 12px;
  width: 620px;
  height: 620px; }

.app-folder-icon {
  padding: 6px;
  spacing-rows: 6px;
  spacing-columns: 6px; }

.app-well-app-running-dot {
  height: 5px;
  width: 5px;
  border-radius: 5px;
  background-color: #eeeeec;
  margin-bottom: 1px; }

.rename-folder-popup .rename-folder-popup-item {
  spacing: 6px; }
  .rename-folder-popup .rename-folder-popup-item:ltr, .rename-folder-popup .rename-folder-popup-item:rtl {
    padding: 0 12px; }

.app-menu,
.app-well-menu {
  max-width: 27.25em; }

.page-indicator {
  padding: 15px 12px 30px; }
  .page-indicator .page-indicator-icon {
    width: 10px;
    height: 10px;
    border-radius: 10px;
    background-color: white; }

.all-apps {
  padding: 0 88px; }

.system-action-icon {
  background-color: rgba(0, 0, 0, 0.8);
  color: #fff;
  border-radius: 99px;
  icon-size: 48px; }

/* Dash */
#dash {
  font-size: 9pt;
  padding: 5px 0;
  border-radius: 0 15px 15px 0;
  border-left-width: 0; }
  #dash:rtl {
    border-radius: 15px 0 0 15px;
    border-right-width: 0; }
  #dash .placeholder {
    background-image: none;
    background-size: contain;
    height: 32px; }
  #dash .empty-dash-drop-target {
    width: 32px;
    height: 32px; }

.dash-item-container > StWidget {
  padding: 5px 10px; }

.dash-label {
  background-color: rgba(32, 32, 32, 0.91);
  border-radius: 7px;
  border: none;
  box-shadow: 0 0 0 1px rgba(255, 255, 255, 0.16);
  color: #eeeeec;
  padding: 6px 8px;
  text-align: center;
  -x-offset: 8px; }

.show-apps .overview-icon {
  color: #eeeeec; }

.show-apps:hover .overview-icon, .show-apps:selected .overview-icon {
  background-color: rgba(238, 238, 236, 0.1); }

.show-apps:focus .overview-icon {
  background-color: rgba(238, 238, 236, 0.3); }

.show-apps:drop .overview-icon {
  background-color: rgba(27, 106, 203, 0.85); }

.show-apps:active .overview-icon, .show-apps:checked .overview-icon {
  background-color: rgba(7, 7, 7, 0.46); }

.show-apps:focus .overview-icon, .show-apps:checked .overview-icon {
  background-color: rgba(7, 7, 7, 0.96);
  color: #eeeeec; }

/* Workspace pager */
.workspace-thumbnails {
  visible-width: 32px;
  spacing: 6px;
  padding: 6px;
  border-radius: 10px 0 0 10px;
  border-right-width: 0; }
  .workspace-thumbnails:rtl {
    border-radius: 0 10px 10px 0;
    border-left-width: 0; }
  .workspace-thumbnails .placeholder {
    background-image: url("resource:///org/gnome/shell/theme/dash-placeholder.svg");
    background-size: contain;
    height: 24px; }

.workspace-thumbnail-indicator {
  border: 3px solid #1b6acb;
  border-radius: 3px;
  padding: 0px; }

.ripple-pointer-location {
  width: 50px;
  height: 50px;
  border-radius: 25px;
  background-color: rgba(143, 187, 240, 0.3);
  box-shadow: 0 0 2px 2px #629fea; }

.pie-timer {
  width: 60px;
  height: 60px;
  -pie-border-width: 3px;
  -pie-border-color: #1b6acb;
  -pie-background-color: rgba(188, 214, 246, 0.3); }

.magnifier-zoom-region {
  border: 2px solid #1b6acb; }
  .magnifier-zoom-region.full-screen {
    border-width: 0; }

.select-area-rubberband {
  background-color: rgba(27, 106, 203, 0.3);
  border: 1px solid #1b6acb; }

.user-icon {
  background-size: contain;
  color: #eeeeec;
  border-radius: 99px;
  icon-size: 4.36em; }
  .user-icon:hover {
    color: white; }
  .user-icon StIcon {
    background-color: rgba(238, 238, 236, 0.05);
    border-radius: 99px;
    padding: 12px;
    width: 2.725em;
    height: 2.725em; }
  .user-icon.user-avatar {
    border: 2px #eeeeec; }

.user-widget.vertical .user-icon {
  icon-size: 6.54em; }
  .user-widget.vertical .user-icon StIcon {
    padding: 20px;
    padding-top: 18px;
    padding-bottom: 22px;
    width: 5.995em;
    height: 5.995em; }

.lightbox {
  background-color: black; }

.flashspot {
  background-color: white; }

.hidden {
  color: rgba(0, 0, 0, 0); }

.caps-lock-warning-label {
  text-align: center;
  padding-bottom: 8px;
  font-size: 10pt;
  color: #f57900; }

/* Tiled window previews */
.tile-preview {
  background-color: rgba(27, 106, 203, 0.5);
  border: 1px solid #1b6acb; }

.tile-preview-left.on-primary {
  border-radius: 6px 0 0 0; }

.tile-preview-right.on-primary {
  border-radius: 0 6px 0 0; }

.tile-preview-left.tile-preview-right.on-primary {
  border-radius: 6px 6px 0 0; }

/* On-screen Keyboard */
#keyboard {
  background-color: rgba(32, 32, 32, 0.9);
  box-shadow: inset 0 1px 0 0 rgba(255, 255, 255, 0.16); }
  #keyboard .page-indicator {
    padding: 6px; }
    #keyboard .page-indicator .page-indicator-icon {
      width: 8px;
      height: 8px; }

.key-container {
  padding: 4px;
  spacing: 4px; }

.keyboard-key {
  color: #eeeeec;
  background-color: #3c3c3c;
  border-color: #2b2b2b;
  box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
  text-shadow: 0 1px rgba(0, 0, 0, 0.2);
  icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
  font-size: 16pt;
  min-height: 1.2em;
  min-width: 1.2em;
  border-width: 1px;
  border-style: solid;
  border-radius: 8px; }
  .keyboard-key:focus {
    color: #eeeeec;
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
    box-shadow: inset 0 0 0 2px rgba(27, 106, 203, 0.6); }
  .keyboard-key:hover, .keyboard-key:checked {
    color: #eeeeec;
    background-color: #424242;
    border-color: #2b2b2b;
    box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2); }
  .keyboard-key:active {
    color: #eeeeec;
    background-color: #2d2d2d;
    border-color: #191919;
    text-shadow: none;
    icon-shadow: none;
    box-shadow: none; }
  .keyboard-key:grayed {
    background-color: #2d2d2d;
    color: #eeeeec;
    border-color: rgba(0, 0, 0, 0.7); }
  .keyboard-key.default-key {
    color: #eeeeec;
    background-color: rgba(45, 45, 45, 0.96);
    border-color: rgba(27, 27, 27, 0.96);
    box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2); }
    .keyboard-key.default-key:hover, .keyboard-key.default-key:checked {
      color: #eeeeec;
      background-color: rgba(50, 50, 50, 0.96);
      border-color: rgba(27, 27, 27, 0.96);
      box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
      text-shadow: 0 1px rgba(0, 0, 0, 0.2);
      icon-shadow: 0 1px rgba(0, 0, 0, 0.2); }
    .keyboard-key.default-key:active {
      color: #eeeeec;
      background-color: rgba(30, 30, 30, 0.96);
      border-color: rgba(9, 9, 9, 0.96);
      text-shadow: none;
      icon-shadow: none;
      box-shadow: none; }
  .keyboard-key.enter-key {
    color: #fff;
    background-color: #1d72d8;
    border-color: #1961b9;
    box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2); }
    .keyboard-key.enter-key:hover, .keyboard-key.enter-key:checked {
      color: #eeeeec;
      background-color: #2c7fe3;
      border-color: #1b68c6;
      box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
      text-shadow: 0 1px rgba(0, 0, 0, 0.2);
      icon-shadow: 0 1px rgba(0, 0, 0, 0.2); }
    .keyboard-key.enter-key:active {
      color: #eeeeec;
      background-color: #185fb4;
      border-color: #134c90;
      text-shadow: none;
      icon-shadow: none;
      box-shadow: none; }
  .keyboard-key.shift-key-uppercase {
    color: #1b6acb; }
  .keyboard-key StIcon {
    icon-size: 1.125em; }

.keyboard-subkeys {
  color: #eeeeec;
  -arrow-border-radius: 10px;
  -arrow-background-color: rgba(32, 32, 32, 0.96);
  -arrow-border-width: 1px;
  -arrow-border-color: rgba(55, 55, 55, 0.96);
  -arrow-base: 20px;
  -arrow-rise: 10px;
  -boxpointer-gap: 6px; }
  .keyboard-subkeys .keyboard-key {
    color: #eeeeec;
    background-color: #3c3c3c;
    border-color: #2b2b2b;
    box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
    text-shadow: 0 1px rgba(0, 0, 0, 0.2);
    icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
    border-radius: 5px; }
    .keyboard-subkeys .keyboard-key:focus {
      color: #eeeeec;
      text-shadow: 0 1px rgba(0, 0, 0, 0.2);
      icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
      box-shadow: inset 0 0 0 2px rgba(27, 106, 203, 0.6); }
    .keyboard-subkeys .keyboard-key:hover, .keyboard-subkeys .keyboard-key:checked {
      color: #eeeeec;
      background-color: #424242;
      border-color: #2b2b2b;
      box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
      text-shadow: 0 1px rgba(0, 0, 0, 0.2);
      icon-shadow: 0 1px rgba(0, 0, 0, 0.2); }
    .keyboard-subkeys .keyboard-key:active {
      color: #eeeeec;
      background-color: #2d2d2d;
      border-color: #191919;
      text-shadow: none;
      icon-shadow: none;
      box-shadow: none; }

.emoji-page .keyboard-key {
  background-color: transparent;
  border: none;
  color: initial; }

.emoji-panel .keyboard-key:latched {
  border-color: #1f76e1;
  background-color: #1b6acb; }

.word-suggestions {
  font-size: 14pt;
  spacing: 12px;
  min-height: 20pt; }

/* Looking Glass */
#LookingGlassDialog {
  background-color: rgba(32, 32, 32, 0.96);
  spacing: 6px;
  padding: 4px;
  border: 1px solid rgba(238, 238, 236, 0.2);
  border-radius: 5px;
  color: #eeeeec; }
  #LookingGlassDialog > #Toolbar {
    border: none;
    border-radius: 5px;
    background-color: rgba(32, 32, 32, 0.96); }
  #LookingGlassDialog .labels {
    spacing: 6px; }
  #LookingGlassDialog .notebook-tab {
    -natural-hpadding: 12px;
    -minimum-hpadding: 6px;
    font-weight: bold;
    color: #cacac4;
    transition-duration: 100ms;
    padding-left: .3em;
    padding-right: .3em;
    border-bottom-width: 2px; }
    #LookingGlassDialog .notebook-tab:hover {
      color: #eeeeec; }
    #LookingGlassDialog .notebook-tab:selected {
      border-bottom-width: 2px;
      box-shadow: inset 0 -2px 0 0 #1f76e1;
      color: #eeeeec; }
  #LookingGlassDialog StBoxLayout#EvalBox {
    padding: 4px;
    spacing: 6px; }
  #LookingGlassDialog StBoxLayout#ResultsArea {
    spacing: 6px; }

.lg-dialog StEntry {
  background-color: rgba(45, 45, 45, 0.56);
  color: #eeeeec;
  border-color: rgba(238, 238, 236, 0.2);
  min-height: 22px;
  selection-background-color: #1b6acb;
  selected-color: #fff; }

.lg-dialog .shell-link {
  color: #629fea; }
  .lg-dialog .shell-link:hover {
    color: #8fbbf0; }
  .lg-dialog .shell-link:active {
    color: #3584e4; }

.lg-dialog .actor-link {
  color: #ccc; }
  .lg-dialog .actor-link:hover {
    color: white; }
  .lg-dialog .actor-link:active {
    color: #999999; }

.lg-completions-text {
  font-size: .9em;
  font-style: italic; }

.lg-obj-inspector-title {
  spacing: 6px; }

.lg-obj-inspector-button {
  border: 1px solid rgba(0, 0, 0, 0.7);
  padding: 4px;
  border-radius: 5px; }
  .lg-obj-inspector-button:hover {
    border: 1px solid #ffffff; }

#lookingGlassExtensions {
  padding: 4px; }

.lg-extensions-list {
  padding: 4px;
  spacing: 6px; }

.lg-extension {
  border: 1px solid rgba(13, 13, 13, 0.7);
  background-color: rgba(45, 45, 45, 0.96);
  border-radius: 5px;
  padding: 4px; }

.lg-extension-name {
  font-weight: bold; }

.lg-extension-meta {
  spacing: 6px; }

#LookingGlassPropertyInspector {
  background: rgba(32, 32, 32, 0.96);
  border: 1px solid rgba(0, 0, 0, 0.7);
  border-radius: 5px;
  padding: 6px; }

/* Login Dialog */
.login-dialog-banner-view {
  padding-top: 24px;
  max-width: 23em; }

.login-dialog,
.unlock-dialog {
  border: none;
  background-color: transparent; }
  .login-dialog StEntry,
  .unlock-dialog StEntry {
    background-color: rgba(45, 44, 46, 0.5);
    color: #eeeeec; }
  .login-dialog .modal-dialog-button-box,
  .unlock-dialog .modal-dialog-button-box {
    spacing: 3px; }
  .login-dialog .modal-dialog-button,
  .unlock-dialog .modal-dialog-button {
    padding: 4px 18px;
    box-shadow: 0 1px 3px rgba(0, 0, 0, 0.2);
    background-color: #5b666a;
    border-color: #5b666a;
    color: #eeeeec; }
    .login-dialog .modal-dialog-button:hover, .login-dialog .modal-dialog-button:focus,
    .unlock-dialog .modal-dialog-button:hover,
    .unlock-dialog .modal-dialog-button:focus {
      background-color: #667478;
      border-color: #667478; }
    .login-dialog .modal-dialog-button:active,
    .unlock-dialog .modal-dialog-button:active {
      box-shadow: none;
      background-color: #4f595d;
      border-color: #4f595d; }
    .login-dialog .modal-dialog-button:insensitive,
    .unlock-dialog .modal-dialog-button:insensitive {
      color: #919190;
      border-color: #252526;
      background-color: #323132;
      box-shadow: none;
      text-shadow: none;
      icon-shadow: none;
      border-color: #4f595d;
      background-color: #4f595d;
      color: rgba(238, 238, 236, 0.7); }
    .login-dialog .modal-dialog-button:default,
    .unlock-dialog .modal-dialog-button:default {
      color: #fff;
      background-color: #1d72d8;
      border-color: #1961b9;
      box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
      text-shadow: 0 1px rgba(0, 0, 0, 0.2);
      icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
      border-color: #1b6acb; }
      .login-dialog .modal-dialog-button:default:hover, .login-dialog .modal-dialog-button:default:focus,
      .unlock-dialog .modal-dialog-button:default:hover,
      .unlock-dialog .modal-dialog-button:default:focus {
        color: #fff;
        background-color: #1f76e1;
        border-color: #1961b9;
        box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.2);
        text-shadow: 0 1px rgba(0, 0, 0, 0.2);
        icon-shadow: 0 1px rgba(0, 0, 0, 0.2);
        background-color: #1f76e1;
        border-color: #1f76e1; }
      .login-dialog .modal-dialog-button:default:active,
      .unlock-dialog .modal-dialog-button:default:active {
        color: #fff;
        background-color: #1963bd;
        border-color: #155099;
        text-shadow: none;
        icon-shadow: none;
        box-shadow: none;
        background-color: #185fb4;
        border-color: #185fb4; }
      .login-dialog .modal-dialog-button:default:insensitive,
      .unlock-dialog .modal-dialog-button:default:insensitive {
        color: #919190;
        border-color: #252526;
        background-color: #323132;
        box-shadow: none;
        text-shadow: none;
        icon-shadow: none;
        border-color: #15539e;
        background-color: #15539e;
        color: rgba(255, 255, 255, 0.7); }
  .login-dialog .cancel-button,
  .login-dialog .switch-user-button,
  .login-dialog .login-dialog-session-list-button,
  .unlock-dialog .cancel-button,
  .unlock-dialog .switch-user-button,
  .unlock-dialog .login-dialog-session-list-button {
    padding: 0;
    border-radius: 99px;
    width: 2.18em;
    height: 2.18em;
    border-color: rgba(53, 53, 53, 0.3);
    background-color: rgba(53, 53, 53, 0.3); }
    .login-dialog .cancel-button StIcon,
    .login-dialog .switch-user-button StIcon,
    .login-dialog .login-dialog-session-list-button StIcon,
    .unlock-dialog .cancel-button StIcon,
    .unlock-dialog .switch-user-button StIcon,
    .unlock-dialog .login-dialog-session-list-button StIcon {
      icon-size: 1.09em; }
  .login-dialog .caps-lock-warning-label,
  .login-dialog .login-dialog-message-warning,
  .unlock-dialog .caps-lock-warning-label,
  .unlock-dialog .login-dialog-message-warning {
    color: #eeeeec; }

.login-dialog-logo-bin {
  padding: 24px 0px; }

.login-dialog-banner {
  color: #d6d6d1; }

.login-dialog-button-box {
  width: 23em;
  spacing: 5px; }

.login-dialog-message {
  text-align: center; }

.login-dialog-message-hint {
  padding-top: 0;
  padding-bottom: 20px; }

.login-dialog-user-selection-box {
  padding: 100px 0px; }

.login-dialog-not-listed-label {
  padding-left: 2px; }
  .login-dialog-not-listed-button:focus .login-dialog-not-listed-label, .login-dialog-not-listed-button:hover .login-dialog-not-listed-label {
    color: #eeeeec; }

.login-dialog-not-listed-label {
  font-size: 10pt;
  font-weight: bold;
  color: #a6a69b;
  padding-top: 1em; }

.login-dialog-user-list-view {
  -st-vfade-offset: 1em; }

.login-dialog-user-list {
  spacing: 12px;
  width: 23em; }
  .login-dialog-user-list:expanded .login-dialog-user-list-item:selected {
    background-color: #1b6acb;
    color: #fff; }
  .login-dialog-user-list:expanded .login-dialog-user-list-item:logged-in {
    border-right: 2px solid #1b6acb; }

.login-dialog-user-list-item {
  border-radius: 9px;
  padding: 6px;
  color: #a6a69b; }
  .login-dialog-user-list-item:ltr .user-widget {
    padding-right: 1em; }
  .login-dialog-user-list-item:rtl .user-widget {
    padding-left: 1em; }
  .login-dialog-user-list-item .login-dialog-timed-login-indicator {
    height: 2px;
    margin-top: 6px;
    background-color: #eeeeec; }
  .login-dialog-user-list-item:focus .login-dialog-timed-login-indicator {
    background-color: #fff; }

.user-widget-label {
  color: #eeeeec; }

.user-widget.horizontal .user-widget-label {
  font-size: 13pt;
  font-weight: bold;
  padding-left: 15px; }
  .user-widget.horizontal .user-widget-label:ltr {
    padding-left: 14px;
    text-align: left; }
  .user-widget.horizontal .user-widget-label:rtl {
    padding-right: 14px;
    text-align: right; }

.user-widget.vertical .user-widget-label {
  font-size: 16pt;
  text-align: center;
  font-weight: normal;
  padding-top: 16px; }

.login-dialog-prompt-layout {
  padding-top: 24px;
  padding-bottom: 12px;
  spacing: 12px;
  width: 23em; }

.login-dialog-prompt-entry {
  height: 1.5em; }

.login-dialog-prompt-label {
  color: #bebeb6;
  font-size: 12pt;
  padding-top: 1em; }

/* Screen Shield */
.unlock-dialog-clock {
  color: white;
  font-weight: 300;
  text-align: center;
  spacing: 24px;
  padding-bottom: 2.5em; }

.unlock-dialog-clock-time {
  font-size: 64pt;
  padding-top: 42px;
  font-feature-settings: "tnum"; }

.unlock-dialog-clock-date {
  font-size: 16pt;
  font-weight: normal; }

.unlock-dialog-clock-hint {
  font-weight: normal;
  padding-top: 48px; }

.unlock-dialog-notifications-container {
  margin: 12px 0;
  spacing: 6px;
  width: 23em;
  background-color: transparent; }
  .unlock-dialog-notifications-container .summary-notification-stack-scrollview {
    padding-top: 0;
    padding-bottom: 0; }
  .unlock-dialog-notifications-container .notification,
  .unlock-dialog-notifications-container .unlock-dialog-notification-source {
    padding: 12px 6px;
    border: none;
    background-color: rgba(32, 32, 32, 0.26);
    color: #eeeeec;
    border-radius: 10px; }
    .unlock-dialog-notifications-container .notification.critical,
    .unlock-dialog-notifications-container .unlock-dialog-notification-source.critical {
      background-color: rgba(32, 32, 32, 0.86); }

.unlock-dialog-notification-label {
  padding: 0px 0px 0px 12px; }

.unlock-dialog-notification-count-text {
  weight: bold;
  padding: 0 6px;
  color: rgba(32, 32, 32, 0.96);
  background-color: rgba(238, 238, 236, 0.3);
  border-radius: 99px;
  margin-right: 12px; }

.screen-shield-background {
  background: black;
  box-shadow: 0px 2px 4px rgba(0, 0, 0, 0.6); }

#lockDialogGroup {
  background-color: #41494c; }

#unlockDialogNotifications StButton#vhandle, #unlockDialogNotifications StButton#hhandle {
  background-color: rgba(53, 53, 53, 0.3); }
  #unlockDialogNotifications StButton#vhandle:hover, #unlockDialogNotifications StButton#vhandle:focus, #unlockDialogNotifications StButton#hhandle:hover, #unlockDialogNotifications StButton#hhandle:focus {
    background-color: rgba(53, 53, 53, 0.5); }
  #unlockDialogNotifications StButton#vhandle:active, #unlockDialogNotifications StButton#hhandle:active {
    background-color: rgba(27, 106, 203, 0.5); }
